/*****************************************************************************
 * Simple program for creating one entry in Alcatel phonebook                *
 *                                                                           *
 * Copyright (c) 2002 Michal Cihar <cihar at email dot cz>                   *
 *                                                                           *
 * This program is  free software; you can  redistribute it and/or modify it *
 * under the terms of the  GNU  General  Public  License as published by the *
 * Free  Software  Foundation; either  version 2 of the License, or (at your *
 * option) any later version.                                                *
 *                                                                           *
 * This code is distributed in the hope that it will  be useful, but WITHOUT *
 * ANY  WARRANTY; without even the  implied  warranty of  MERCHANTABILITY or *
 * FITNESS FOR A  PARTICULAR PURPOSE. See the GNU General Public License for *
 * more details.                                                             *
 *                                                                           *
 * In addition to GNU GPL this code may be used also in non GPL programs but *
 * if and  only if  programmer/distributor  of that  code  receives  written *
 * permission from author of this code.                                      *
 *                                                                           *
 *****************************************************************************/
/* $Id: main.c,v 1.18 2002/05/08 08:56:16 michal Exp $ */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>
#include <unistd.h>
#define _GNU_SOURCE
#include <getopt.h>
#include <libgen.h>
#include <string.h>

#include "modem.h"
#include "mobile_info.h"
#include "logging.h"
#include "version.h"
#include "charset.h"
#include "sms.h"
#include "common.h"
#include "pdu.h"
#include "alcatel.h"

char *progname;

char default_device[] = "/dev/ttyS1";
char default_lock[] = "/var/lock/LCK..%s";
char default_init[] = "AT S7=45 S0=0 L1 V1 X4 &c1 E1 Q0";
int default_rate = 19200;
int wanted_extra_params = 1;
char **extra_params;
int binary_mode_active = 0;

extern char alc_contacts_field_names[ALC_CONTACTS_FIELDS][20];

void help() {
    printf("This is demo program for " ALCASYNC_NAME " version " ALCASYNC_VERSION "\nCopyright (c) " ALCASYNC_COPYRIGHT "\n");
    printf("Usage:  %s [options] created_item_name\n", basename(progname));
    printf("Options:\n");
    printf("    -d<dev>/--device=<dev> ... modem device [%s]\n",default_device);
    printf("    -K<fn>/--lock=<fn> ... lock filename [/var/lock/LCK..device]\n");
    printf("    -I<s>/-init=<s> ... init string for modem [%s]\n",default_init);
    printf("    -B<n>/--rate=<n> ... baud rate [%d]\n",default_rate);
    printf("    -q/--quiet ... be more quiet\n");
    printf("    -v/--verbose ... be more verbose\n");
    exit(255);
}

void version() {
    printf("This is demo program for " ALCASYNC_NAME " version " ALCASYNC_VERSION "\nCopyright (c) " ALCASYNC_COPYRIGHT "\n");
    printf("This program is free software; you can redistribute it and/or modify\n");
    printf("it under the terms of the GNU General Public License as published by\n");
    printf("the Free Software Foundation; either version 2 of the License, or\n");
    printf("(at your option) any later version.\n");
    exit(255);
}


void parse_params(int argc, char *argv[]) {
    int result, i;
    char *devname;
    
static const struct option longopts[]={
{"rate"             ,1,0,'B'},
{"init"             ,1,0,'I'},
{"lock"             ,1,0,'K'},
{"device"           ,1,0,'d'},
    
{"quiet"            ,0,0,'q'},
{"verbose"          ,0,0,'v'},

{"help"             ,0,0,'h'},
{"version"          ,0,0,'V'},
{NULL               ,0,0,0  }};


    progname = argv[0];
    /* set some dafaults */    
    strcpy(device,default_device);
    devname = strrchr(device, '/');
    devname++;
    sprintf(lockname, default_lock, devname);
    strcpy(initstring,default_init);
    rate=default_rate;

    while ((result=getopt_long(argc,argv,"bcatCThd:K:I:B:is::SvqVr:x:lTmw::",longopts,NULL))>0) {
        switch (result) {
            case 'V': 
                version(); 
                break;;
            case 'h': 
                help(); 
                break;;
            case 'd':
                strcpy(device,optarg);
                devname = strrchr(device, '/');
                if (devname == NULL) {
                    message(MSG_ERROR,"Bad device name!");
                    exit(202);
                }
                devname++;
                sprintf(lockname, default_lock, devname);
                break;
            case 'K':
                strcpy(lockname,optarg);
                break;
            case 'I':
                strcpy(initstring,optarg);
                break;
            case 'B':
                rate = atoi(optarg);
                break;
            case 'q':
                if (msg_level<MSG_NONE) msg_level++;
                break;
            case 'v':
                if (msg_level>MSG_ALL) msg_level--;
                break;
        }
        
    }
    
    if (argc - optind < wanted_extra_params) {
        message(MSG_ERROR,"Not enough addional parameters!");
        exit(203);
    } else if (argc - optind > wanted_extra_params) {
        message(MSG_ERROR,"Too much optional parameters! %d will be ignored.", argc - optind - wanted_extra_params);
    }

    if (optind < argc) {
        extra_params = (char **)malloc(MIN((argc - optind),wanted_extra_params) * sizeof(char *));
        i = 0;
        while ((optind + i)< argc) {
            extra_params[i] = argv[optind + i];
            i++;
        }
    }    
    
    switch (rate) {
        case 2400:   baudrate=B2400; break;
        case 4800:   baudrate=B4800; break;
        case 9600:   baudrate=B9600; break;
        case 19200:  baudrate=B19200; break;
        case 38400:  baudrate=B38400; break;        
        default: 
            message(MSG_ERROR,"Ivalid baud rate (%d)!", rate);
            exit(201);
    }
}

void shorten_extra_params(int count) {
    wanted_extra_params -= count;
    memmove(extra_params, extra_params + count, wanted_extra_params * sizeof(char *)); 
}

void shutdown() {
    if (binary_mode_active) {
        alcatel_detach();
        alcatel_done();
    }
    modem_close();
}

void print_hex(char *buffer) {
    static char conv[] = "0123456789ABCDEF";
    int i;
    for (i=1; i<=buffer[0]; i++)
        printf ("%c%c (%c) ", conv[((char)buffer[i] & 0xff) >> 4], conv[buffer[i] & 0x0f], isprint(buffer[i])?buffer[i]:'*');
}

void create_alc_item(alc_type sync, alc_type type, char *name) {
    alc_type *result;
    int i, j;
    int *ids, *items;
    AlcatelFieldStruct *field;
    int fieldno;

    int count=100; /* default to high cat number... */
    
    switch (sync) {
        case ALC_SYNC_CALENDAR:
            count = ALC_CALENDAR_FIELDS;
            break;
        case ALC_SYNC_TODO:
            count = ALC_TODO_FIELDS;
            break;
        case ALC_SYNC_CONTACTS:
            count = ALC_CONTACTS_FIELDS;
            break;
    }
                
    alcatel_attach();
    
    alcatel_start_session();
    if (alcatel_select_type(type)) {
        alcatel_begin_transfer(sync);

        for (int fieldno = 0; fieldno < contact; fieldno++) {
            AlcatelFieldStruct *field = (*it).getField(fieldno);
            if (field != NULL) {
                if (field->data != NULL) {
                    alcatel_create_field(ALC_SYNC_TYPE_TODO, fieldno, field);
                    free(field->data);
                }
                free(field);
            }
        }

        int num = alcatel_commit(ALC_SYNC_TYPE_TODO);
        if (num == -1) {
            KMessageBox::error(win, i18n("Creating of todo failed!"), i18n("Error"));
        } else {
            (*it).Created = false;
            (*it).Modified = false;
            (*it).Storage = StorageMobile;
            (*it).PrevStorage = StorageNone;
            (*it).Id = num;
            ::message(MSG_INFO, "Created: %s (id = %d)", (*it).getName().latin1(), num);
        }
    } else {
        message(MSG_ERROR, "Can not open sync session!");
    }
    
    alcatel_close_session(type);
    alcatel_detach();
}

void print_message(MessageData* sms) {
    if (((*sms).stat == MESSAGE_SENT) || ((*sms).stat == MESSAGE_UNSENT)) {
        printf("To: %s\nSMSC: %s\nStatus: %d\nPosition: %d\n\n%s\n",
            (*sms).sendr,
            (*sms).smsc,
            (*sms).stat,
            (*sms).pos,
            (*sms).ascii);
    } else {
        printf("From: %s\nDate: %sSMSC: %s\nStatus: %d\nPosition: %d\n\n%s\n",
            (*sms).sendr,
            ctime(&((*sms).date)),
            (*sms).smsc,
            (*sms).stat,
            (*sms).pos,
            (*sms).ascii);
    }
}

void list_messages() {
    MessageData *sms;
    int i;
    sms = get_messages(MESSAGE_ALL);
    i = 0;
    while (sms[i].pos != -1) {
        print_message(sms+i);
        i++;
        if (sms[i].pos != -1) printf ("-------------------------------------------------------------------------------\n");
    }
    free(sms);
}

void read_messages() {
    MessageData *sms;
    int i;
    for (i = 0; i < action_read_pos; i++){
        sms = get_message(action_read[i]);
        if (sms) {
            print_message(sms);
            free(sms);
        } else {
            message(MSG_ERROR,"Message %d not found!", action_read[i]);
        }
        if (i != action_read_pos - 1) printf ("-------------------------------------------------------------------------------\n");
    }
}

void delete_messages() {
    int i;
    for (i = 0; i < action_del_pos; i++){
        if (delete_message(action_del[i])) {
            message(MSG_INFO,"Message %d deleted!", action_del[i]);
        } else {
            message(MSG_ERROR,"Message %d not found!", action_del[i]);
        }
    }
}

void send_message() {
    char pdu[1024];
    int i;
    
    make_pdu(extra_params[0], extra_params[1], 0, PDU_CLASS_SIM, pdu);
    i = send_message(pdu);
    if (i != -1) printf ("Message send, message reference = %d\n", i);
    else message(MSG_ERROR, "Message not sent!");
    shorten_extra_params(2);
}

void write_message() {
    char pdu[1024];
    int i;

    make_pdu(extra_params[0], extra_params[1], 0, PDU_CLASS_SIM, pdu);
    i = put_message(pdu, (action_write_type == 0) ? MESSAGE_SENT : MESSAGE_UNSENT);
    if (i != -1) printf ("Message written at position %d\n", i);
    else message(MSG_ERROR, "Message not written!");
    shorten_extra_params(2);
}

int main(int argc, char *argv[]) {
    char data[1024];
    char *s;
    int i, j;

    msg_level = MSG_INFO;

    atexit(shutdown);
    
    parse_params(argc,argv);

    message(MSG_INFO, "This is test program for " ALCASYNC_NAME " version " ALCASYNC_VERSION);
    message(MSG_INFO, "Copyright (c) " ALCASYNC_COPYRIGHT);

    if (!modem_open()) {
        switch (modem_errno) {
            case ERR_MDM_LOCK:
                message(MSG_ERROR, "Modem locked!");
                exit(1);
                break;
            case ERR_MDM_OPEN:
                message(MSG_ERROR, "Modem can't be opened!");
                exit(2);
                break;
            default:
                message(MSG_ERROR, "Unknown error!");
                exit(5);
                break;
        }
    }
    modem_setup(); 

    if (!modem_init()) {
        switch (modem_errno) {
            case ERR_MDM_PDU:
                message(MSG_ERROR, "Failed selecting PDU mode!");
                exit(4);
                break;
            case ERR_MDM_AT:
                message(MSG_ERROR, "Modem not reacting!");
                exit(3);
                break;
            default:
                message(MSG_ERROR, "Unknown error!");
                exit(5);
                break;
        }
    }
    if (action_info) {
        get_manufacturer(data,sizeof(data));
        printf ("Manufacturer: %s\n",data);
        
        get_model(data,sizeof(data));
        printf ("Model: %s\n",data);
        
        get_revision(data,sizeof(data));
        printf ("Revision: %s\n",data);
        
        get_sn(data,sizeof(data));
        printf ("Serial no: %s\n",data);

        get_battery(&i,&j);
        printf ("Batery status: %d%%, state: %d\n", j, i);
        
        get_signal(&i,&j);
        printf ("Signal strenght: %s (%d), error rate: %d\n",i == 99 ? "unknown" : mobil_signal_info[i], i, j);

        s = get_smsc();
        printf ("SMS centre number: %s\n", s);
    }

    if (action_message) {
        if (action_del_pos > 0) {
            delete_messages();
            action_implicit = 0;
        }
        if (action_read_pos > 0) {
            read_messages();
            action_implicit = 0;
        }
        if (action_write) {
            write_message();
            action_implicit = 0;
        }
        if (action_send) {
            send_message();
            action_implicit = 0;
        }
        if (action_list || action_implicit) list_messages();
    }
    
    if (action_monitor) {
        while (1) {
            get_signal(&i,&j);
            printf ("Signal strenght: %s (%d), error rate: %d\n",i == 99 ? "unknown" : mobil_signal_info[i],i,j);
            usleep(100000);
        }
    }

    if (action_binary) {
        alcatel_init();
        binary_mode_active = 1;

        if (delete_todo_cats) del_alc_cats(ALC_SYNC_TODO, ALC_SYNC_TYPE_TODO, ALC_LIST_TODO_CAT);
        if (delete_contacts_cats) del_alc_cats(ALC_SYNC_CONTACTS, ALC_SYNC_TYPE_CONTACTS, ALC_LIST_CONTACTS_CAT);


        if (create_todo_cat != NULL) create_alc_cat(ALC_SYNC_TODO, ALC_SYNC_TYPE_TODO, ALC_LIST_TODO_CAT, create_todo_cat);
        if (create_contacts_cat != NULL) create_alc_cat(ALC_SYNC_CONTACTS, ALC_SYNC_TYPE_CONTACTS, ALC_LIST_CONTACTS_CAT, create_contacts_cat);
        if (action_todo_cat) list_alc_cats(ALC_SYNC_TODO, ALC_SYNC_TYPE_TODO, ALC_LIST_TODO_CAT);
        if (action_todo) list_alc_items(ALC_SYNC_TODO, ALC_SYNC_TYPE_TODO);
        if (action_contacts_cat) list_alc_cats(ALC_SYNC_CONTACTS, ALC_SYNC_TYPE_CONTACTS, ALC_LIST_CONTACTS_CAT);
        if (action_contacts) list_alc_items(ALC_SYNC_CONTACTS, ALC_SYNC_TYPE_CONTACTS);
        if (action_calendar) list_alc_items(ALC_SYNC_CALENDAR, ALC_SYNC_TYPE_CALENDAR);
        
        alcatel_done();
        binary_mode_active = 0;
        
    }


    return 0;
}
